@@ -299,7 +299,6 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 	d(printf("ecbe_open_calendar(%p, %p, %sonly if exists, user=%s, pass=%s)\n", backend, cal, only_if_exists?"":"not ", username?username:"(null)", password?password:"(null)"));
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
-	printf ("uri of cal to open is : %s\n", uristr);
 
 	g_mutex_lock (cbex->priv->open_lock);
 
@@ -391,22 +390,20 @@ remove_calendar (ECalBackendSync *backend, EDataCal *cal)
 	ExchangeAccountFolderResult result;
 	const char *uri;
 
-	/* FIXME: Deleting calendar/tasks from respective views */
 	uri = e_folder_exchange_get_internal_uri (cbex->folder);
 	result = exchange_account_remove_folder (cbex->account, uri);
-	return GNOME_Evolution_Calendar_Success;
-
-	/* status = e_folder_exchange_delete (cbex->folder, NULL);
-	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
-		d(printf ("successfully removed\n"));
+	if (result == EXCHANGE_ACCOUNT_FOLDER_OK)
 		return GNOME_Evolution_Calendar_Success;
-	} else if (status == E2K_HTTP_UNAUTHORIZED) {
-		d(printf ("permission denied\n"));
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST)
+		return GNOME_Evolution_Calendar_NoSuchCal;
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_UNSUPPORTED_OPERATION)
 		return GNOME_Evolution_Calendar_PermissionDenied;
-	} else {
-		d(printf ("other error\n"));
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_OFFLINE)
+		return GNOME_Evolution_Calendar_OfflineUnavailable;
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED)
+		return GNOME_Evolution_Calendar_PermissionDenied;
+	else
 		return GNOME_Evolution_Calendar_OtherError;
-	} */
 }
 
 static void
@@ -498,7 +495,7 @@ e_cal_backend_exchange_add_object (ECalBackendExchange *cbex,
 	const char *uid;
 	struct icaltimetype rid;
 
-	d(printf("ecbe_add_object(%p, %s, %s)\n", cbex, href, lastmod));
+	printf("ecbe_add_object(%p, %s, %s)\n", cbex, href, lastmod);
 
 	uid = icalcomponent_get_uid (comp);
 	ecomp = g_hash_table_lookup (cbex->priv->objects, uid);
@@ -540,6 +537,8 @@ e_cal_backend_exchange_add_object (ECalBackendExchange *cbex,
 
 			ecomp->instances = g_list_prepend (ecomp->instances,
 						   icalcomponent_new_clone (comp));
+			if (ecomp->icomp)
+				e_cal_util_remove_instances (ecomp->icomp, rid, CALOBJ_MOD_THIS);
 		}
 	} else
 		ecomp->icomp = icalcomponent_new_clone (comp);
@@ -560,9 +559,14 @@ discard_detached_instance (ECalBackendExchangeComponent *ecomp,
 		if (icaltime_compare (inst_rid, rid) == 0) {
 			ecomp->instances = g_list_remove (ecomp->instances, inst->data);
 			icalcomponent_free (inst->data);
-			return;
+			break;
 		}
 	}
+
+	if (ecomp->icomp) /* This check should ideally not be needed. */
+		e_cal_util_remove_instances (ecomp->icomp, rid, CALOBJ_MOD_THIS);
+
+	return;
 }
 
 /**
@@ -669,35 +673,94 @@ get_exchange_comp (ECalBackendExchange *cbex, const char *uid)
 	return NULL;			
 }
 
+static void
+add_instances_to_vcal (gpointer value, gpointer user_data)
+{
+	icalcomponent *recurrence = value;
+	icalcomponent *vcalendar = user_data;
+
+	icalcomponent_add_component (
+		vcalendar,
+		icalcomponent_new_clone (recurrence));		     
+
+}
+
 static ECalBackendSyncStatus
 get_object (ECalBackendSync *backend, EDataCal *cal,
 	    const char *uid, const char *rid, char **object)
 {
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	ECalBackendExchangeComponent *ecomp;
-	ECalComponent *comp;
 	
 	d(printf("ecbe_get_object(%p, %p, uid=%s, rid=%s)\n", backend, cal, uid, rid));
 	
 	g_return_val_if_fail (uid != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	/*any other asserts?*/
 	
+	*object = NULL;
 	ecomp = g_hash_table_lookup (cbex->priv->objects, uid);
-	if ((!ecomp) || (!ecomp->icomp))
+	if (!ecomp)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
-	
-	/*anything on recur id here????*/
-	
-	comp = e_cal_component_new ();
-	if (e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp))) {
-		*object = e_cal_component_get_as_string (comp);
-	} else {
-		*object = NULL;
-		g_object_unref (comp);
+
+	if (!rid && (!(ecomp->icomp)))	
 		return GNOME_Evolution_Calendar_ObjectNotFound;
+
+	if (rid && *rid) {
+		GList *l;
+		struct icaltimetype inst_rid, key_rid;
+		gboolean inst_found = FALSE;
+
+		for (l = ecomp->instances; l ; l = l->next) {
+			key_rid = icaltime_from_string (rid);
+			inst_rid = icalcomponent_get_recurrenceid (l->data);
+			if (icaltime_compare (inst_rid, key_rid) == 0) {
+				inst_found = TRUE;
+				break;
+			}
+		}
+		
+		if (inst_found) {
+			*object = g_strdup (icalcomponent_as_ical_string (l->data));
+		} else {
+			/* Instance is not found. Send the master object instead */	
+			if (ecomp->icomp) {
+				icalcomponent *new_inst;
+				struct icaltimetype itt;
+
+				itt = icaltime_from_string (rid);
+				new_inst = e_cal_util_construct_instance (
+					icalcomponent_new_clone (ecomp->icomp), itt);
+				if (!new_inst)
+					return GNOME_Evolution_Calendar_ObjectNotFound;
+				
+				*object = g_strdup (icalcomponent_as_ical_string (new_inst));
+				icalcomponent_free (new_inst);
+			} else {
+				/* Oops. No instance and no master as well !! */
+				return GNOME_Evolution_Calendar_ObjectNotFound;
+			} /* Close check for master object */
+		} /* Close check for instance */
+	}/* Close check if rid is being asked */
+	else {
+		/* rid is not asked */
+		if (g_list_length (ecomp->instances) > 0) {
+			/* Ok, we have detached instances, return a VCALENDAR */
+			/* Send the whole object including all its detatched instances */
+			icalcomponent *vcalcomp;
+
+			vcalcomp = e_cal_util_new_top_level ();
+			icalcomponent_add_component (vcalcomp,
+				icalcomponent_new_clone (ecomp->icomp));
+			g_list_foreach (ecomp->instances, (GFunc) add_instances_to_vcal, vcalcomp);
+			
+			*object = g_strdup (icalcomponent_as_ical_string (vcalcomp));
+			icalcomponent_free (vcalcomp);	
+		} else {
+			/* There are no detached instances. Send only the master object */
+			*object = g_strdup (icalcomponent_as_ical_string (ecomp->icomp));
+		}
 	}
-	
-	g_object_unref (comp);
+
 	return GNOME_Evolution_Calendar_Success;
 }
 
